/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QVBoxLayout>
#include <QSplitter>
#include <QPlainTextEdit>
#include <QTextCursor>
#include <QList>
#include <QFont>

#include "analysiswidget.h"

TAnalysisWidget::TAnalysisWidget(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QSplitter							*splitter_widget;
	QVBoxLayout							*widget_vlayout;
	QList<int>							splitter_sizes;
	QFont								text_font;

	text_font.setFamily(QStringLiteral("Courier New"));
	text_font.setFixedPitch(true);

	widget_vlayout = new QVBoxLayout(this);

	splitter_widget = new QSplitter(Qt::Vertical,this);
	widget_vlayout->addWidget(splitter_widget);
	
	d_analysis_graph_widget = new TGraphWidget();
	d_analysis_graph_widget->setMinimumSize(0,100);
	d_analysis_graph_widget->SetGraphType(TGraphWidget::BAR_GRAPH);
	d_analysis_graph_widget->SetHLegendMode(TGraphWidget::DEFAULT_LEGEND_PRECISION);
	d_analysis_graph_widget->SetVLegendMode(TGraphWidget::INTEGER_LEGEND_PRECISION);
	splitter_widget->addWidget(d_analysis_graph_widget);
	
	d_analysis_text_edit = new QPlainTextEdit();
	d_analysis_text_edit->setMinimumSize(0,0);
	d_analysis_text_edit->setFont(text_font);
	d_analysis_text_edit->setLineWrapMode(QPlainTextEdit::NoWrap);
	d_analysis_text_edit->setReadOnly(true);
	splitter_widget->addWidget(d_analysis_text_edit);

	// approximate sizes
	splitter_sizes.push_back(200);
	splitter_sizes.push_back(200);

	splitter_widget->setSizes(splitter_sizes);
	
	d_analysis_graph_widget->HorizontalLegend("MCG Arm Length");
	d_analysis_graph_widget->VerticalLegend("Quantity");
	d_analysis_graph_widget->Title("Distribution and Tolerance");
	d_analysis_graph_widget->EnableMouseCoordinateDisplay(false);

}

TAnalysisWidget::~TAnalysisWidget(void)
{
}

void TAnalysisWidget::Clear(void)
{
	d_analysis_graph_widget->DeleteDataSets();
	d_analysis_text_edit->clear();
	
	d_analysis_graph_widget->ScaleToFit();
}

int TAnalysisWidget::Create_Graph_Set(
	const QColor						&color,
	const TGraphWidget::TGraphType		type)
{
	return d_analysis_graph_widget->CreateDataSet(color,type);
}

void TAnalysisWidget::Add_Point_Graph(
	const int							&set,
	const TVector2						&pnt)
{
	d_analysis_graph_widget->AddPoint(set,pnt);
}

void TAnalysisWidget::Scale_To_Fit_Graph(void)
{
	d_analysis_graph_widget->ScaleToFit();
}

void TAnalysisWidget::Add_Text(
	const QString						&text)
{
	d_analysis_text_edit->appendPlainText(text);
}

void TAnalysisWidget::Add_Text_End(void)
{
	d_analysis_text_edit->moveCursor(QTextCursor::Start);
}

