/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef MCGHEADERFILE
#define MCGHEADERFILE

#include <QMainWindow>
#include <QList>

#include "measurement.h"

class QTabWidget;
class QLabel;
class QDragEnterEvent;
class QDropEvent;
class QSettings;
class QCloseEvent;
class QResizeEvent;
class QMoveEvent;
class QAction;
class QTimer;

class TAnalysisWidget;
class TToleranceWidget;
class TOpenGLDisplayWidget;
class TReportWidget;
class TPathWidget;
class TSelectionDialog;
class TMessageBox;


class TMachineCheckingGauge : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TMachineCheckingGauge(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TMachineCheckingGauge(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	void New(void);
	void Open(void);
	void Open_Recent(void);
	void Save(void);
	void Import(void);
	void Export(void);
	void Output(void);
	void Help_About(void);
	
	void Tab_Index_Changed(int);
	void Update_Path(void);
	void Tolerance_Changed(void);
	void MCG_Selection(const double&,const double&,const double&,const int);
	
	void Simulation_Start(void);
	void Simulation_Stop(void);
	void Simulation_Timeout(void);
	void Generate_DMIS_Program(void);
	void Draw_MCG_Item(const int);
	void Arm_Length_Changed(const double&);
	
protected:
	virtual void dragEnterEvent(QDragEnterEvent *DragEnterEvent);
	virtual void dropEvent(QDropEvent *DropEvent);
	virtual void closeEvent(QCloseEvent *event);

private:
	
enum TFileType
{
	FILE_TYPE_UNKNOWN = 0,
	FILE_TYPE_MCG,
	FILE_TYPE_XML,
	FILE_TYPE_TXT,
	FILE_TYPE_DMO
};
	
	QAction								*d_new_action;
	QAction								*d_open_action;
	QAction								*d_save_action;
	QAction								*d_export_action;
	QAction								*d_import_action;
	QAction								*d_quit_action;
	QAction								*d_report_zoomin_action;
	QAction								*d_report_zoomout_action;
	QAction								*d_report_output_action;
	QAction								*d_report_print_action;
	QAction								*d_about_action;
	QTabWidget							*d_tab_widget;
	QTimer								*d_simulation_timer;
	TAnalysisWidget						*d_analysis_widget;
	TSelectionDialog					*d_selection_dialog;
	TOpenGLDisplayWidget				*d_opengl_display_widget;
	TToleranceWidget					*d_tolerance_widget;
	TReportWidget						*d_report_widget;
	TPathWidget							*d_path_widget;
	TMessageBox							*d_msg_box;
	QSettings							*d_settings;
	
	TMeasurement						d_measurement;
	QString								d_active_file_name;
	QList<QAction*>						d_recent_file_actions;
	QStringList							d_recent_files;
	bool								d_data_changed;
	bool								d_position_selection_dialog;

	QString								d_data_path;
	QString								d_file_path;
	std::vector<TVector3>				d_motion_path;
	int									d_simulation_index;
			
	void Open_File(const QString&);
	void Save_File(const QString&);
	void Import_XML_Measurement(const QString&);
	void Import_DMO_Measurement(const QString&);
	void Import_Text_Measurement(const QString&);
	void Export_Measurement(const QString&);

	void Update_Display(void);
	void Update_Recent_File_Actions(void);
	
	TFileType Identify_File_Type(const QString &file_name) const;
	
	void Load_Recent_File_List(const QString &file_path);
	void Save_Recent_File_List(const QString &file_path) const;


// NOT IMPLEMENTED
	TMachineCheckingGauge(const TMachineCheckingGauge&);
	TMachineCheckingGauge& operator=(const TMachineCheckingGauge&);
};

#endif
