/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef OPENGLDISPLAYHEADERFILE
#define OPENGLDISPLAYHEADERFILE

#include <QFrame>
#include <vector>

#include "../../core/openglwidget.h"
#include "../../core/openglvertexbuffer.h"
#include "../../core/openglgeometryfactory.h"

class QOpenGLShaderProgram;
class TOpenGLVertexBuffer;

class TOpenGLDisplay : public TOpenGLWidget
{
	Q_OBJECT
public:

struct TMCGGauge
{
	int									id;
	bool								selected;
	bool								error;
	
	TVector3							min_xyz;
	TVector3							max_xyz;

	TVector3							center;
	TVector3							axis;
	double								length;
};


// CREATORS
	TOpenGLDisplay(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TOpenGLDisplay(void);

// ACCESSORS
	int MCG_Count(void) const {return 0;}

// MANIPULATORS
	void Define_Machine_Volume(const TVector3 &pnt_min, const TVector3 &pnt_max);
	
	void Clear_Render_Items(void);
	void Clear_Render_Volume(void);
	
	void Add_MCG(const TMCGGauge &mcg,const bool single_arm);
	
	void Set_MCG_Path(const std::vector<TVector3> &path_points);
	void Clear_MCG_Path(void);

	void Clear_Selection(void);
	void Set_Selection(const int id);

protected:

	virtual void GLInit(void);
	virtual void GLResize(const int width, const int height);
	virtual void GLRender(void);

private:
	
struct TRenderMCGItem
{
	int									id;
	bool								selected;
	bool								error;
	
	int									mcg_vertex_id;
	int									mcg_nominal_id;
	GLenum								mode;
	TVector3							color;
};

	double								d_alpha;
	
	TVector3							d_machine_min;
	TVector3							d_machine_max;
	TVector3							d_axis_draw_pos;
	double								d_axis_draw_len;
	
	TOpenGLVertexBuffer					d_opengl_vertex_buffer;
	
	QOpenGLShaderProgram				*d_triangle_shader_program;
	QOpenGLShaderProgram				*d_basic_shader_program;
	
	GLint								d_triangle_uniform_color;
	GLint								d_triangle_uniform_specular;
	GLint								d_triangle_uniform_offset;
	GLint								d_triangle_attribute_vertex;
	GLint								d_triangle_attribute_normal;
	
	GLint								d_basic_uniform_color;
	GLint								d_basic_attribute_vertex;

	std::vector<TRenderMCGItem>			d_mcg_base_items;
	std::vector<TRenderMCGItem>			d_mcg_arm_items;

	int									d_machine_volume_vertex_id;
	int									d_mcg_path_id;
		
	void Render_Triangle_Buffer(const unsigned int size,const TVector3 &offset,const TVector3 &color, const double &specular,const double &opacity, double * const vertex,double * const nominal);
	void Render_Triangle_Buffer(const GLenum mode,const unsigned int size,const TVector3 &offset,const TVector3 &color, const double &specular,const double &opacity, double * const vertex,double * const nominal);	void Render_Line_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Line_Strip_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Point_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);


// NOT IMPLEMENTED
	TOpenGLDisplay(const TOpenGLDisplay&);
	TOpenGLDisplay& operator=(const TOpenGLDisplay&);
};

#endif

