/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QFrame>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QToolButton>
#include <QSpacerItem>
#include <QSpinBox>
#include <QSlider>
#include <QComboBox>
#include <QFile>
#include <QFileDialog>
#include <QSizePolicy>
#include <assert.h>

#include "pathwidget.h"

TPathWidget::TPathWidget(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*widget_layout;
	QGridLayout							*button_layout;
	QFrame								*separator_hline;
	QLabel								*azimuth_count_label;
	QLabel								*azimuth_end_label;
	QLabel								*azimuth_start_label;
	QLabel								*elevation_count_label;
	QLabel								*elevation_end_label;
	QLabel								*elevation_start_label;
	QLabel								*options_pattern_count_label;
	QLabel								*options_point_count_label;
	QSpacerItem							*widget_vspacer;
	QSizePolicy							sizepolicy_minimum_fixed(QSizePolicy::Minimum, QSizePolicy::Fixed);
	
	sizepolicy_minimum_fixed.setHorizontalStretch(0);
	sizepolicy_minimum_fixed.setVerticalStretch(0);
 
	widget_layout = new QGridLayout(this);
	widget_layout->setSpacing(6);

	elevation_start_label = new QLabel(this);
	widget_layout->addWidget(elevation_start_label,0,0,1,1);

	d_elevation_start_spin = new QDoubleSpinBox(this);
	d_elevation_start_spin->setAlignment(Qt::AlignCenter);
	d_elevation_start_spin->setDecimals(1);
	d_elevation_start_spin->setMinimum(-45);
	d_elevation_start_spin->setMaximum(-5);
	d_elevation_start_spin->setSingleStep(5);
	d_elevation_start_spin->setValue(-40);
	widget_layout->addWidget(d_elevation_start_spin,0,1,1,1);

	elevation_end_label = new QLabel(this);
	widget_layout->addWidget(elevation_end_label,1,0,1,1);

	d_elevation_end_spin = new QDoubleSpinBox(this);
	d_elevation_end_spin->setAlignment(Qt::AlignCenter);
	d_elevation_end_spin->setDecimals(1);
	d_elevation_end_spin->setMinimum(5);
	d_elevation_end_spin->setMaximum(45);
	d_elevation_end_spin->setSingleStep(5);
	d_elevation_end_spin->setValue(40);
	widget_layout->addWidget(d_elevation_end_spin,1,1,1,1);

	elevation_count_label = new QLabel(this);
	widget_layout->addWidget(elevation_count_label,2,0,1,1);

	d_elevation_count_spin = new QSpinBox(this);
	d_elevation_count_spin->setAlignment(Qt::AlignCenter);
	d_elevation_count_spin->setMinimum(2);
	d_elevation_count_spin->setMaximum(91);
	widget_layout->addWidget(d_elevation_count_spin,2,1,1,1);

	azimuth_start_label = new QLabel(this);
	widget_layout->addWidget(azimuth_start_label,3,0,1,1);

	d_azimuth_start_spin = new QDoubleSpinBox(this);
	d_azimuth_start_spin->setEnabled(false);
	d_azimuth_start_spin->setAlignment(Qt::AlignCenter);
	d_azimuth_start_spin->setDecimals(1);
	d_azimuth_start_spin->setMaximum(360);
	widget_layout->addWidget(d_azimuth_start_spin,3,1,1,1);

	azimuth_end_label = new QLabel(this);
	widget_layout->addWidget(azimuth_end_label,4,0,1,1);

	d_azimuth_end_spin = new QDoubleSpinBox(this);
	d_azimuth_end_spin->setEnabled(false);
	d_azimuth_end_spin->setAlignment(Qt::AlignCenter);
	d_azimuth_end_spin->setDecimals(1);
	d_azimuth_end_spin->setMaximum(360);
	d_azimuth_end_spin->setValue(360);
	widget_layout->addWidget(d_azimuth_end_spin,4,1,1,1);

	azimuth_count_label = new QLabel(this);
	widget_layout->addWidget(azimuth_count_label,5,0,1,1);

	d_azimuth_count_spin = new QSpinBox(this);
	d_azimuth_count_spin->setAlignment(Qt::AlignCenter);
	d_azimuth_count_spin->setMinimum(3);
	d_azimuth_count_spin->setMaximum(360);
	d_azimuth_count_spin->setValue(8);
	widget_layout->addWidget(d_azimuth_count_spin,5,1,1,1);
	
	options_point_count_label = new QLabel(this);
	widget_layout->addWidget(options_point_count_label,6,0,1,1);

	d_options_point_count_spin = new QSpinBox(this);
	d_options_point_count_spin->setAlignment(Qt::AlignCenter);
	d_options_point_count_spin->setMinimum(1);
	d_options_point_count_spin->setValue(2);
	widget_layout->addWidget(d_options_point_count_spin,6,1,1,1);

	options_pattern_count_label = new QLabel(this);
	widget_layout->addWidget(options_pattern_count_label,7,0,1,1);
	
	d_options_pattern_count_spin = new QSpinBox(this);
	d_options_pattern_count_spin->setAlignment(Qt::AlignCenter);
	d_options_pattern_count_spin->setMinimum(1);
	d_options_pattern_count_spin->setValue(2);
	widget_layout->addWidget(d_options_pattern_count_spin,7,1,1,1);
	
	d_option_horizontal_arm_check = new QCheckBox(this);
	widget_layout->addWidget(d_option_horizontal_arm_check,9,0,1,2);

	d_horizontal_arm_axis_combo = new QComboBox(this);
	widget_layout->addWidget(d_horizontal_arm_axis_combo,10,0,1,2);
	
	d_arm_length_combo = new QComboBox(this);
	widget_layout->addWidget(d_arm_length_combo,11,0,1,2);
	
	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline,12,0,1,2);
	
	button_layout = new QGridLayout();
	button_layout->setContentsMargins(0,0,0,0);
	
	d_simulation_button = new QToolButton(this);
	sizepolicy_minimum_fixed.setHeightForWidth(d_simulation_button->sizePolicy().hasHeightForWidth());
	d_simulation_button->setSizePolicy(sizepolicy_minimum_fixed);
	button_layout->addWidget(d_simulation_button,0,0,1,1);

	d_simulator_slider = new QSlider(Qt::Horizontal,this);
	button_layout->addWidget(d_simulator_slider,0,1,1,1);
	
	d_create_dmis_button = new QToolButton(this);
	sizepolicy_minimum_fixed.setHeightForWidth(d_create_dmis_button->sizePolicy().hasHeightForWidth());
	d_create_dmis_button->setSizePolicy(sizepolicy_minimum_fixed);
	button_layout->addWidget(d_create_dmis_button,1,0,1,1);

	d_create_path_button = new QToolButton(this);
	sizepolicy_minimum_fixed.setHeightForWidth(d_create_path_button->sizePolicy().hasHeightForWidth());
	d_create_path_button->setSizePolicy(sizepolicy_minimum_fixed);
	button_layout->addWidget(d_create_path_button,2,0,1,1);
	
	widget_layout->addLayout(button_layout,13,0,1,2);
	
	widget_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	widget_layout->addItem(widget_vspacer,14,0,1,2);

	QWidget::setTabOrder(d_elevation_start_spin,d_elevation_end_spin);
	QWidget::setTabOrder(d_elevation_end_spin,d_elevation_count_spin);
	QWidget::setTabOrder(d_elevation_count_spin,d_azimuth_start_spin);
	QWidget::setTabOrder(d_azimuth_start_spin,d_azimuth_end_spin);
	QWidget::setTabOrder(d_azimuth_end_spin,d_azimuth_count_spin);
	QWidget::setTabOrder(d_azimuth_count_spin,d_options_point_count_spin);
	QWidget::setTabOrder(d_options_point_count_spin,d_options_pattern_count_spin);
	QWidget::setTabOrder(d_options_pattern_count_spin,d_option_horizontal_arm_check);
	QWidget::setTabOrder(d_option_horizontal_arm_check,d_horizontal_arm_axis_combo);
	QWidget::setTabOrder(d_horizontal_arm_axis_combo,d_arm_length_combo);
	QWidget::setTabOrder(d_arm_length_combo,d_simulation_button);
	QWidget::setTabOrder(d_simulation_button,d_simulator_slider);
	QWidget::setTabOrder(d_simulator_slider,d_create_dmis_button);
	QWidget::setTabOrder(d_create_dmis_button,d_create_path_button);
	
	// defaults
	d_horizontal_arm_axis_combo->setEnabled(false);
	d_simulation_running = false;

	elevation_start_label->setText(QStringLiteral("Starting Elevation:"));
	elevation_count_label->setText(QStringLiteral("Elevation Count:"));
	elevation_end_label->setText(QStringLiteral("Ending Elevation:"));
	azimuth_start_label->setText(QStringLiteral("Starting Azimuth:"));
	azimuth_end_label->setText(QStringLiteral("Ending Azimuth:"));
	azimuth_count_label->setText(QStringLiteral("Azimuth Count:"));
	options_point_count_label->setText(QStringLiteral("Measurement Point Count:"));
	d_option_horizontal_arm_check->setText(QStringLiteral("Horizontal Arm Mode"));
	options_pattern_count_label->setText(QStringLiteral("Pattern Repeat Count:"));
	d_simulation_button->setText(QStringLiteral("Run Simulation"));
	d_create_dmis_button->setText(QStringLiteral("Create DMIS Program"));
	d_create_path_button->setText(QStringLiteral("Create Path File"));
	
	d_horizontal_arm_axis_combo->addItem(QStringLiteral("Arm Extends Plus Y"),TPathWidget::HORIZONTAL_ARM_AXIS_YPLUS);
	d_horizontal_arm_axis_combo->addItem(QStringLiteral("Arm Extends Minus Y"),TPathWidget::HORIZONTAL_ARM_AXIS_YMINUS);
	d_horizontal_arm_axis_combo->addItem(QStringLiteral("Arm Extends Plus X"),TPathWidget::HORIZONTAL_ARM_AXIS_XPLUS);
	d_horizontal_arm_axis_combo->addItem(QStringLiteral("Arm Extends Minus X"),TPathWidget::HORIZONTAL_ARM_AXIS_XMINUS);

	d_arm_length_combo->addItem(QStringLiteral("Arm Length 101 mm"),TPathWidget::ARM_LENGTH_101);
	d_arm_length_combo->addItem(QStringLiteral("Arm Length 151 mm"),TPathWidget::ARM_LENGTH_151);
	d_arm_length_combo->addItem(QStringLiteral("Arm Length 226 mm"),TPathWidget::ARM_LENGTH_226);
	d_arm_length_combo->addItem(QStringLiteral("Arm Length 380 mm"),TPathWidget::ARM_LENGTH_380);
	d_arm_length_combo->addItem(QStringLiteral("Arm Length 532 mm"),TPathWidget::ARM_LENGTH_532);
	d_arm_length_combo->addItem(QStringLiteral("Arm Length 685 mm"),TPathWidget::ARM_LENGTH_685);

	connect(d_elevation_start_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_elevation_end_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_azimuth_start_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_azimuth_end_spin,static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	
	connect(d_elevation_count_spin,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_azimuth_count_spin,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_options_point_count_spin,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	connect(d_options_pattern_count_spin,static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),this,&TPathWidget::Update_Path);
	
	connect(d_option_horizontal_arm_check,&QCheckBox::toggled,this,&TPathWidget::Parameter_Changed);
	connect(d_horizontal_arm_axis_combo,static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),this,&TPathWidget::Parameter_Changed);

	connect(d_simulator_slider,&QSlider::valueChanged,this,&TPathWidget::Simulate_Position);
	
	connect(d_simulation_button,&QToolButton::clicked,this,&TPathWidget::Simulation_Clicked);
	connect(d_create_dmis_button,&QToolButton::clicked,this,&TPathWidget::Generate_DMIS_Program);
	connect(d_create_path_button,&QToolButton::clicked,this,&TPathWidget::Create_Path);
	connect(d_arm_length_combo,static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),this,&TPathWidget::Arm_Selection_Changed);
}

TPathWidget::~TPathWidget(void)
{
}

std::vector<TVector3> TPathWidget::Motion_Path(
	const TVector3						&position,
	const double						&arm_length)
{
	std::vector<TPathPoint>::const_iterator iter;
	std::vector<TVector3>				path;
	TVector3							pt;
	TVector3							last_pt,p2,p3;
	bool								init(true);
	static const double					MOTION_OFFSET(25);
		
	this->Generate_Path();
	
	for(iter = d_path_points.begin();iter != d_path_points.end();++iter)
	{
		if((*iter).point_type == TPathWidget::MOVE_POINT)
		{
			pt = position + (*iter).point_vector * (arm_length + MOTION_OFFSET);
			
			if(init)
			{
				last_pt = pt;
				init = false;
			}
			else
			{
				p2 = (pt + last_pt) / 2.0;
				p3 = (p2 + last_pt) / 2.0;
				
				path.push_back(p3);
				path.push_back(p2);
				
				p3 = (p2 + pt) / 2.0;

				path.push_back(p3);
				path.push_back(pt);
				
				last_pt = pt;
			}
			
		}
		else
		{
			pt = position + (*iter).point_vector * (arm_length + MOTION_OFFSET);
			path.push_back(pt);

			pt = position + (*iter).point_vector * arm_length;
			path.push_back(pt);
			
			pt = position + (*iter).point_vector * (arm_length + MOTION_OFFSET);
			path.push_back(pt);
			
			last_pt = pt;
		}
	}
		
	d_simulator_slider->setRange(0,path.size()-1);
	
	return path;
}

std::vector<TPathWidget::TPathPoint> TPathWidget::Path_Points(void)
{		
	this->Generate_Path();
	return d_path_points;
}

bool TPathWidget::Horizontal_Arm_Checked(void) const
{
	return d_option_horizontal_arm_check->isChecked();
}

TPathWidget::TArmLength TPathWidget::Arm_Length_Selection(void) const
{
	return static_cast<TPathWidget::TArmLength>(d_arm_length_combo->currentData().toInt());
}

double TPathWidget::Arm_Length(void) const
{
	TPathWidget::TArmLength				arm_length;
	
	arm_length = static_cast<TPathWidget::TArmLength>(d_arm_length_combo->currentData().toInt());
	
	switch(arm_length)
	{
		case TPathWidget::ARM_LENGTH_101:
			return 101.0;
			
		case TPathWidget::ARM_LENGTH_151:
			return 151.0;
			
		case TPathWidget::ARM_LENGTH_226:
			return 226.0;
			
		case TPathWidget::ARM_LENGTH_380:
			return 380.0;
			
		case TPathWidget::ARM_LENGTH_532:
			return 532.0;
			
		case TPathWidget::ARM_LENGTH_685:
			return 685.0;
	}
	
	return 101.0;
}

double TPathWidget::Elevation_Start(void) const
{
	return d_elevation_start_spin->value();
}

double TPathWidget::Elevation_End(void) const
{
	return d_elevation_end_spin->value();
}

int TPathWidget::Azimuth_Count(void) const
{
	return d_azimuth_count_spin->value();
}

int TPathWidget::Elevation_Count(void) const
{
	return d_elevation_count_spin->value();
}

int TPathWidget::Pattern_Count(void) const
{
	return d_options_pattern_count_spin->value();
}

int TPathWidget::Point_Count(void) const
{
	return d_options_point_count_spin->value();
}

int TPathWidget::Arm_Axis(void) const
{
	return d_horizontal_arm_axis_combo->currentData().toInt();
}

void TPathWidget::Set_Horizontal_Arm_Checked(
	const bool							state)
{
	d_option_horizontal_arm_check->setChecked(state);
	d_horizontal_arm_axis_combo->setEnabled(state);
}

void TPathWidget::Set_Arm_Length_Selection(
	const TPathWidget::TArmLength		arm_length)
{
	int									index;
	
	index = d_arm_length_combo->findData(static_cast<int>(arm_length));
	
	if(!(index < 0))
	{
		d_arm_length_combo->setCurrentIndex(index);
	}
	else
	{
		d_arm_length_combo->setCurrentIndex(0);
	}
}

void TPathWidget::Set_Elevation_Start(
	const double						&value)
{
	d_elevation_start_spin->setValue(value);
}

void TPathWidget::Set_Elevation_End(
	const double						&value)
{
	d_elevation_end_spin->setValue(value);
}

void TPathWidget::Set_Azimuth_Count(
	const int							count)
{
	d_azimuth_count_spin->setValue(count);
}

void TPathWidget::Set_Elevation_Count(
	const int							count)
{
	d_elevation_count_spin->setValue(count);
}

void TPathWidget::Set_Pattern_Count(
	const int							count)
{
	d_options_pattern_count_spin->setValue(count);
}

void TPathWidget::Set_Point_Count(
	const int							count)
{
	d_options_point_count_spin->setValue(count);
}

void TPathWidget::Set_Arm_Axis(
	const int							index)
{
	switch(index)
	{
		case TPathWidget::HORIZONTAL_ARM_AXIS_YPLUS:
		default:
			d_horizontal_arm_axis_combo->setCurrentIndex(0);
			break;
			
		case TPathWidget::HORIZONTAL_ARM_AXIS_YMINUS:
			d_horizontal_arm_axis_combo->setCurrentIndex(1);
			break;
			
		case TPathWidget::HORIZONTAL_ARM_AXIS_XPLUS:
			d_horizontal_arm_axis_combo->setCurrentIndex(2);
			break;
			
		case TPathWidget::HORIZONTAL_ARM_AXIS_XMINUS:
			d_horizontal_arm_axis_combo->setCurrentIndex(3);
			break;
	}
}

void TPathWidget::Set_Simulator_Slider_Position(
	const int 							index)
{
	bool								prev_state;
	
	prev_state = d_simulator_slider->blockSignals(true);
	d_simulator_slider->setValue(index);
	d_simulator_slider->blockSignals(prev_state);
}

void TPathWidget::End_Simulation(void)
{
	if(d_simulation_running)
	{
		d_simulation_running = false;
		d_simulation_button->setText(QStringLiteral("Run Simulation"));

		emit Stop_Simulation();
		
		d_simulator_slider->setEnabled(true);
	}
}

void TPathWidget::Parameter_Changed(void)
{
	d_horizontal_arm_axis_combo->setEnabled(d_option_horizontal_arm_check->isChecked());
	
	emit Update_Path();
}

void TPathWidget::Create_Path(void)
{
	QString								file_name;
	QString								file_filters;
	QFile								file;
	std::vector<TPathWidget::TPathPoint>::const_iterator iter;
	
	file_filters = QString("MCG Measurement Path (*.path)");
	
	file_name = QFileDialog::getSaveFileName(this,
											 QStringLiteral("Input name of path file to create"),
											 QString(),
											 file_filters);
	
	if(file_name.length())
	{
		this->Generate_Path();
		
		file.setFileName(file_name);
		
		if(file.open(QIODevice::WriteOnly))
		{
			file.write("MCG_PATH_FILE_V1\r\n");
			
			for(iter = d_path_points.begin();iter != d_path_points.end();++iter)
			{
				if((*iter).point_type == TPathWidget::MOVE_POINT)
				{
					file.write(QString("P %1 %2 %3\r\n").arg((*iter).point_vector.i,0,'f',9).arg((*iter).point_vector.j,0,'f',9).arg((*iter).point_vector.k,0,'f',9).toLatin1());
				}
				else if((*iter).point_type == TPathWidget::MEASURE_POINT)
				{
					file.write(QString("M %1 %2 %3\r\n").arg((*iter).point_vector.i,0,'f',9).arg((*iter).point_vector.j,0,'f',9).arg((*iter).point_vector.k,0,'f',9).toLatin1());
				}
			}

			file.close();
		}
	}
}

void TPathWidget::Simulation_Clicked(void)
{
	if(d_simulation_running)
	{
		d_simulation_running = false;
		d_simulation_button->setText(QStringLiteral("Run Simulation"));

		emit Stop_Simulation();
		
		d_simulator_slider->setEnabled(true);
	}
	else
	{
		d_simulation_running = true;
		d_simulation_button->setText(QStringLiteral("Stop Simulation"));
		
		d_simulator_slider->setEnabled(false);

		emit Start_Simulation();
	}
}

void TPathWidget::Arm_Selection_Changed(void)
{
	double								arm_length;
	
	arm_length = this->Arm_Length();
	emit Arm_Length_Changed(arm_length);
}

void TPathWidget::Generate_Path(void)
{
	TPathPoint							path_point;
	TPathPoint							path_point_previous;
	double								elevation_increment;
	double								azimuth_increment;
	int									elevation_cntr;
	int									azimuth_cntr;
	int									pattern_cntr;
	int									point_cntr;
	double								elevation;
	double								azimuth;
	bool								point_accessible;
	static const double					DEGTORAD(0.01745329251994329577);
	static const double					PI2(6.28318530717958647692);
		
	d_path_points.clear();
	
	assert(d_elevation_count_spin->value() > 1);
	assert(d_elevation_end_spin->value() > d_elevation_start_spin->value());
	assert(d_azimuth_count_spin->value() > 2);
	
	elevation_increment = d_elevation_end_spin->value() - d_elevation_start_spin->value();
	elevation_increment /= static_cast<double>(d_elevation_count_spin->value() - 1);
	elevation_increment *= DEGTORAD;
	
	azimuth_increment = PI2 / static_cast<double>(d_azimuth_count_spin->value());
	
	// starting azimuth is always +x
	path_point.point_type = TPathWidget::MOVE_POINT;
	path_point.point_vector.Set(1.0,0.0,0.0);
	d_path_points.push_back(path_point);
	
	path_point_previous = path_point;
	
	for(pattern_cntr = 0;pattern_cntr < d_options_pattern_count_spin->value();++pattern_cntr)
	{
		elevation = d_elevation_start_spin->value() * DEGTORAD;
		
		for(elevation_cntr = 0;elevation_cntr < d_elevation_count_spin->value();++elevation_cntr)
		{
			azimuth = d_azimuth_start_spin->value();
			
			for(azimuth_cntr = 0;azimuth_cntr < d_azimuth_count_spin->value();++azimuth_cntr)
			{
				path_point.point_vector.Set(cos(azimuth) * cos(elevation),sin(azimuth) * cos(elevation),sin(elevation));
				path_point.point_vector = this->Adjust_Move_Target(path_point.point_vector,&point_accessible);
				
				this->Generate_Intermediate_Move_Points(path_point_previous.point_vector,path_point.point_vector);
				
				path_point.point_type = TPathWidget::MOVE_POINT;
				d_path_points.push_back(path_point);
				
				if(point_accessible)	// only measure points that are accessible
				{
					for(point_cntr = 0;point_cntr < d_options_point_count_spin->value();++point_cntr)
					{
						path_point.point_type = TPathWidget::MEASURE_POINT;
						d_path_points.push_back(path_point);
					}
					
					path_point.point_type = TPathWidget::MOVE_POINT;
					d_path_points.push_back(path_point);
				}
				
				path_point_previous = path_point;
				
				azimuth += azimuth_increment;
			}
			
			elevation += elevation_increment;
		}
	}
	
	path_point.point_type = TPathWidget::MOVE_POINT;
	path_point.point_vector.Set(1,0,0);
	
	this->Generate_Intermediate_Move_Points(path_point_previous.point_vector,path_point.point_vector);
	
	d_path_points.push_back(path_point);
}

TVector3 TPathWidget::Adjust_Move_Target(
	const TVector3						&pnt,
	bool								* const point_accessible) const
{
	TVector3							pnt_xy;
	TVector3							pnt_return;
	TVector3							arm_axis;
	double								angle_pnt;
	double								safe_z_position;
	int									arm_axis_value;
	static const double					AVOIDANCE_ANGLE(0.707106781186548);			// cos(45)
	
	pnt_return = pnt;
	
	if(point_accessible)
	{
		*point_accessible = true;
	}
	
	arm_axis_value = d_horizontal_arm_axis_combo->currentData().toInt();
	
	// arm_axis vector is the side the arm cannot measure points
	switch(arm_axis_value)
	{
		case TPathWidget::HORIZONTAL_ARM_AXIS_YPLUS:
		default:
			arm_axis.Set(0.0,1.0,0.0);
			break;
		
		case TPathWidget::HORIZONTAL_ARM_AXIS_YMINUS:
			arm_axis.Set(0.0,-1.0,0.0);
			break;
			
		case TPathWidget::HORIZONTAL_ARM_AXIS_XPLUS:
			arm_axis.Set(1.0,0.0,0.0);
			break;
			
		case TPathWidget::HORIZONTAL_ARM_AXIS_XMINUS:
			arm_axis.Set(-1.0,0.0,0.0);
			break;
	}
	
	if(pnt.z < 0 && d_option_horizontal_arm_check->isChecked() == true)
	{
		pnt_xy = pnt;
		pnt_xy.z = 0;
		
		if(pnt_xy.Length())
		{
			pnt_xy.Normal();
	
			angle_pnt = arm_axis ^ pnt_xy;	// shadow is assumed to be on +Y side of coordinate system
			
			safe_z_position = -AVOIDANCE_ANGLE * (angle_pnt - 1) / (AVOIDANCE_ANGLE - 1.0);
			
			if(angle_pnt > AVOIDANCE_ANGLE && pnt.z < safe_z_position)
			{
				pnt_return = pnt_xy;
				pnt_return *= sqrt(1 - safe_z_position * safe_z_position);
				pnt_return.z = safe_z_position;
				
				if(point_accessible)
				{
					*point_accessible = false;
				}
			}
		}
	}
	
	return pnt_return;
}

void TPathWidget::Generate_Intermediate_Move_Points(
	const TVector3 						&start,
	const TVector3 						&end)
{
	double								angle_start_end;
	TPathPoint							path_point;
	TVector3							pnt;
	static const double					MAX_MOVE_ANGLE(0.965925826289068);			// cos(15) degrees
	static const double					COS45(0.707106781186548);
	static const double					OPPOSING_LIMIT(-0.99984769515639123916);	// cannot use opposing points where dot = -1.
	
	angle_start_end = start ^ end;
	
	assert(angle_start_end > OPPOSING_LIMIT);	// points cannot be opposite to eachother.  best to blow up program if this odd event happens
	
	if(angle_start_end > MAX_MOVE_ANGLE)
	{
		return;
	}
	
	pnt = (start + end)/2;
	pnt.Normal();
	
	// All moves must stay within +/- 45 of horizontal plane
	if(pnt.z < -COS45)
	{
		pnt.z = 0;
		pnt.Normal();
		
		pnt.z = -COS45;
		pnt.x *= COS45;
		pnt.y *= COS45;
	}
	
	if(pnt.z > COS45)
	{
		pnt.z = 0;
		pnt.Normal();
		
		pnt.z = COS45;
		pnt.x *= COS45;
		pnt.y *= COS45;
	}
	
	pnt = this->Adjust_Move_Target(pnt);
	
	this->Generate_Intermediate_Move_Points(start,pnt);
	
	path_point.point_vector = pnt;
	path_point.point_type = TPathWidget::MOVE_POINT;
	d_path_points.push_back(path_point);
	
	this->Generate_Intermediate_Move_Points(pnt,end);
}


