/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef PATHWIDGETHEADERFILE
#define PATHWIDGETHEADERFILE

#include <QWidget>
#include <vector>

#include "../../core/vector3.h"

class QCheckBox;
class QDoubleSpinBox;
class QToolButton;
class QSpinBox;
class QSlider;
class QComboBox;
class QLabel;

class TPathWidget : public QWidget
{
Q_OBJECT

public:
	
enum THorizontalArmAxis
{
	HORIZONTAL_ARM_AXIS_UNDEFINED = 0,
	HORIZONTAL_ARM_AXIS_YPLUS,
	HORIZONTAL_ARM_AXIS_YMINUS,
	HORIZONTAL_ARM_AXIS_XPLUS,
	HORIZONTAL_ARM_AXIS_XMINUS
};
	
enum TArmLength
{
	ARM_LENGTH_101 = 0,
	ARM_LENGTH_151,
	ARM_LENGTH_226,
	ARM_LENGTH_380,
	ARM_LENGTH_532,
	ARM_LENGTH_685
};

enum TPathPointType
{
	MOVE_POINT=0,
	MEASURE_POINT
};

struct TPathPoint
{
	TVector3							point_vector;
	TPathPointType						point_type;
};
	
// CREATORS
	TPathWidget(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TPathWidget(void);

// ACCESSORS
	std::vector<TVector3> Motion_Path(const TVector3 &position,const double &arm_length);
	std::vector<TPathPoint> Path_Points(void);
	
	bool Horizontal_Arm_Checked(void) const;
	TArmLength Arm_Length_Selection(void) const;
	double Arm_Length(void) const;
	double Elevation_Start(void) const;
	double Elevation_End(void) const;
	int Azimuth_Count(void) const;
	int Elevation_Count(void) const;
	int Pattern_Count(void) const;
	int Point_Count(void) const;
	int Arm_Axis(void) const;

// MANIPULATORS
	
	void Set_Horizontal_Arm_Checked(const bool state);
	void Set_Arm_Length_Selection(const TArmLength arm_length);
	void Set_Elevation_Start(const double &value);
	void Set_Elevation_End(const double &value);
	void Set_Azimuth_Count(const int);
	void Set_Elevation_Count(const int);
	void Set_Pattern_Count(const int);
	void Set_Point_Count(const int);
	void Set_Arm_Axis(const int);
	
	void Set_Simulator_Slider_Position(const int index);
	void End_Simulation(void);

signals:
	void Update_Path(void);
	void Simulate_Position(const int);
	void Start_Simulation(void);
	void Stop_Simulation(void);
	void Arm_Length_Changed(const double&);
	void Generate_DMIS_Program(void);

private slots:
	void Parameter_Changed(void);
	void Create_Path(void);
	void Simulation_Clicked(void);
	void Arm_Selection_Changed(void);

private:

	QCheckBox							*d_option_horizontal_arm_check;
	QDoubleSpinBox						*d_azimuth_end_spin;
	QDoubleSpinBox						*d_azimuth_start_spin;
	QDoubleSpinBox						*d_elevation_end_spin;
	QDoubleSpinBox						*d_elevation_start_spin;
	QToolButton							*d_simulation_button;
	QToolButton							*d_create_path_button;
	QToolButton							*d_create_dmis_button;
	QSpinBox							*d_azimuth_count_spin;
	QSpinBox							*d_elevation_count_spin;
	QSpinBox							*d_options_pattern_count_spin;
	QSpinBox							*d_options_point_count_spin;
	QSlider								*d_simulator_slider;
	QComboBox							*d_horizontal_arm_axis_combo;
	QComboBox							*d_arm_length_combo;

	std::vector<TPathPoint>				d_path_points;
	bool								d_simulation_running;
//	bool								d_path_update_required;

	void Generate_Path(void);

	TVector3 Adjust_Move_Target(const TVector3 &pnt,bool * const point_accessible=0) const;
	void Generate_Intermediate_Move_Points(const TVector3 &start,const TVector3 &end);


// NOT IMPLEMENTED
	TPathWidget(const TPathWidget&);
	TPathWidget& operator=(const TPathWidget&);
};

#endif
