/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef REPORTWIDGETHEADERFILE
#define REPORTWIDGETHEADERFILE

#include <QWidget>
#include <QRect>
#include <QSize>
#include <QPoint>
#include <QPen>
#include <QBrush>

#include "../../core/vector3.h"

class QGraphicsView;
class QGraphicsScene;
class QGraphicsItem;
class QGraphicsTextItem;
class QGraphicsRectItem;
class QPainter;
//class QToolButton;

class TReportWidget : public QWidget
{
	Q_OBJECT
public:
	// CREATORS
	TReportWidget(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TReportWidget(void);
	
	// ACCESSORS
	QString Font_Family_Name(void) const {return d_font_family_name;}
	
	// MANIPUATORS
	void Set_Date_Time(const QString &Text);
	void Set_Temperature(const double &value);
	void Set_Tolerance(const double &value);
	void Set_Measurement_Range(const double &value);
	
	void Set_Pivot_Sphere_Start(const TVector3 &position);
	void Set_Pivot_Sphere_End(const TVector3 &position);
	void Set_Measurement_Sphere(const TVector3 &position);
	
	void Set_Probe_Offset(const TVector3 &offset);
	void Set_Probe_Error(const double &value);

	void Set_Average_Length_XY(const double &value,const bool valid);
	void Set_Average_Length_YZ(const double &value,const bool valid);
	void Set_Average_Length_ZX(const double &value,const bool valid);
	void Set_Relative_Scale_Error_X(const double &value,const bool valid);
	void Set_Relative_Scale_Error_Y(const double &value,const bool valid);
	void Set_Relative_Scale_Error_Z(const double &value,const bool valid);
	void Set_Squareness_Error_XY(const double &value,const bool valid);
	void Set_Squareness_Error_YZ(const double &value,const bool valid);
	void Set_Squareness_Error_ZX(const double &value,const bool valid);

	void Set_Average_Measurement_Length(const double &value);
	void Set_Minimum_Measured_Length(const double &value);
	void Set_Maximum_Measured_Length(const double &value);
	void Set_Median_Measured_Length(const double &value);
	void Set_Stddev_Measured_Length(const double &value);
	
	void Clear_Report(void);
	
	void Add_Measurement_Data(
			const double &elevation,
			const double &azimuth,
			const double &length,
			const double &length_range,
			const double &length_stddev,
			const double &length_dev);
			
	void Update_Report(void);
			
	
public slots:
	void Zoom_In(void);
	void Zoom_Out(void);
	void Write(const QString &file_name);
	void Print(void);

private:
	
struct TMeasurementData
{
	QGraphicsTextItem					*elevation_value;
	QGraphicsTextItem					*angle_value;
	QGraphicsTextItem					*length_value;
	QGraphicsTextItem					*length_range_value;
	QGraphicsTextItem					*length_stddev_value;
	QGraphicsTextItem					*length_dev_value;
};

struct TReportBase
{
	TReportBase(void);
	virtual ~TReportBase(void);
	
	QGraphicsTextItem					*title_text;
	QGraphicsTextItem					*units_text;
	QGraphicsTextItem					*page_text;
	QGraphicsTextItem					*program_information_text;
	QGraphicsRectItem 					*page_background_rect;
	QGraphicsRectItem 					*header_line_rect;
	QGraphicsRectItem 					*footer_line_rect;
	QRect								page_rect;
	QRect								page_content_rect;
	int									column_left;
	int									column_right;
};

struct TReportFirstPage : public TReportBase
{
	TReportFirstPage(void);
	virtual ~TReportFirstPage(void);
	
	QGraphicsTextItem					*test_results_label;
	QGraphicsRectItem 					*test_results_rect;
	QGraphicsTextItem					*test_date_time_label;
	QGraphicsTextItem					*test_date_time_value;
	QGraphicsTextItem					*temperature_label;
	QGraphicsTextItem					*temperature_value;
	QGraphicsTextItem					*range_label;
	QGraphicsTextItem					*range_value;
	QGraphicsTextItem					*tolerance_label;
	QGraphicsTextItem					*tolerance_value;

	QGraphicsTextItem					*test_position_label;
	QGraphicsRectItem 					*test_position_rect;
	QGraphicsTextItem					*pivot_sphere_start_label;
	QGraphicsTextItem					*pivot_sphere_start_value;
	QGraphicsTextItem					*pivot_sphere_end_label;
	QGraphicsTextItem					*pivot_sphere_end_value;
	QGraphicsTextItem					*measured_sphere_label;
	QGraphicsTextItem					*measured_sphere_value;
	
	QGraphicsTextItem					*probe_results_label;
	QGraphicsRectItem 					*probe_results_rect;
	QGraphicsTextItem					*probe_offset_label;
	QGraphicsTextItem					*probe_offset_value;
	QGraphicsTextItem					*probe_error_label;
	QGraphicsTextItem					*probe_error_value;
	
	QGraphicsTextItem					*analysis_results_label;
	QGraphicsRectItem 					*analysis_results_rect;
	QGraphicsTextItem					*average_length_xy_label;
	QGraphicsTextItem					*average_length_xy_value;
	QGraphicsTextItem					*average_length_yz_label;
	QGraphicsTextItem					*average_length_yz_value;
	QGraphicsTextItem					*average_length_zx_label;
	QGraphicsTextItem					*average_length_zx_value;
	QGraphicsTextItem					*relative_scale_error_x_label;
	QGraphicsTextItem					*relative_scale_error_x_value;
	QGraphicsTextItem					*relative_scale_error_y_label;
	QGraphicsTextItem					*relative_scale_error_y_value;
	QGraphicsTextItem					*relative_scale_error_z_label;
	QGraphicsTextItem					*relative_scale_error_z_value;
	QGraphicsTextItem					*squareness_xy_label;
	QGraphicsTextItem					*squareness_xy_value;
	QGraphicsTextItem					*squareness_yz_label;
	QGraphicsTextItem					*squareness_yz_value;
	QGraphicsTextItem					*squareness_zx_label;
	QGraphicsTextItem					*squareness_zx_value;

	QGraphicsTextItem					*statistic_results_label;
	QGraphicsRectItem 					*statistics_results_rect;
	QGraphicsTextItem					*average_length_label;
	QGraphicsTextItem					*average_length_value;
	QGraphicsTextItem					*minimum_length_label;
	QGraphicsTextItem					*minimum_length_value;
	QGraphicsTextItem					*maximum_length_label;
	QGraphicsTextItem					*maximum_length_value;
	QGraphicsTextItem					*median_length_label;
	QGraphicsTextItem					*median_length_value;
	QGraphicsTextItem					*stddev_length_label;
	QGraphicsTextItem					*stddev_length_value;
};

struct TReportMeasurementPage : public TReportBase
{
	TReportMeasurementPage(void);
	virtual ~TReportMeasurementPage(void);
	
	QGraphicsTextItem					*elevation_label;
	QGraphicsTextItem					*azimuth_label;
	QGraphicsTextItem					*length_label;
	QGraphicsTextItem					*length_range_label;
	QGraphicsTextItem					*length_stddev_label;
	QGraphicsTextItem					*length_deviation_label;
	QGraphicsRectItem 					*label_line_rect;
};

	
	QGraphicsView 						*d_graphic_view;
	QGraphicsScene						*d_graphic_scene;
    double								d_display_scale_factor;
	
	QBrush								d_graphic_view_brush;
	QString								d_font_family_name;
	int									d_font_size;
	int									d_font_size_title;
	int									d_font_size_header;
	int									d_font_size_footer;
	
	QSize								d_page_size;
	double								d_page_margin;
	double								d_line_height;
	
	TReportFirstPage					d_first_page;
	std::vector<TReportMeasurementPage>	d_measurement_pages;
	std::vector<TMeasurementData>		d_measurement_data;	
	
	void Initialize_Page(TReportBase * const page,const QPoint &top_left);
	void Setup_Base_Page(TReportBase * const page);
	void Setup_Summary_Page(TReportFirstPage * const page);
	void Setup_Measurement_Page(TReportMeasurementPage * const page,const int index_start,const int index_end);
	
	bool Render_Page(QPainter * const painter,const QRect &page_rect,const int page_number,bool * const is_last_page);
	
	// NOT IMPLEMENTED
	TReportWidget(const TReportWidget&);
	TReportWidget& operator=(const TReportWidget&);
};

#endif


