/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFont>
#include <QPlainTextEdit>
#include <QVBoxLayout>
#include <QTextCursor>

#include "selectiondialog.h"

TSelectionDialog::TSelectionDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFont								font;
	QVBoxLayout							*dialog_vlayout;

	font.setFamily(QStringLiteral("Courier New"));

	this->resize(318,203);

	dialog_vlayout = new QVBoxLayout(this);

	d_selection_edit = new QPlainTextEdit(this);
	d_selection_edit->setFont(font);
	d_selection_edit->setToolTipDuration(10);
	d_selection_edit->setLineWrapMode(QPlainTextEdit::NoWrap);
	d_selection_edit->setReadOnly(true);
	dialog_vlayout->addWidget(d_selection_edit);

	this->setWindowTitle(QStringLiteral("Selection Information"));
}

TSelectionDialog::~TSelectionDialog(void)
{
}

void TSelectionDialog::Clear_Text(void)
{
	d_selection_edit->clear();
}

void TSelectionDialog::Add_Text(
	const QString 						&text)
{
	d_selection_edit->appendPlainText(text);
}

void TSelectionDialog::Add_Text_End(void)
{
	QTextCursor							text_cursor;
	
	text_cursor = d_selection_edit->textCursor();
	text_cursor.setPosition(0);	// beginning of document
	
	d_selection_edit->setTextCursor(text_cursor);
}
