/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef TOLERANCWIDGETHEADERFILE
#define TOLERANCWIDGETHEADERFILE

#include <QWidget>

class QLineEdit;

class TToleranceWidget : public QWidget
{
	Q_OBJECT
public:
// CREATORS
	TToleranceWidget(const QWidget *Parent=0,Qt::WindowFlags Flags=Qt::WindowFlags());
	virtual ~TToleranceWidget(void);

// ACCESSORS
	
	QString Tolerance_Expression(void) const;

	double Tolerance_Item_A(void) const {return d_tolerance_a;}
	double Tolerance_Item_B(void) const {return d_tolerance_b;}
	double Tolerance_Item_C(void) const {return d_tolerance_c;}
	double Tolerance_Item_D(void) const {return d_tolerance_d;}
	double Tolerance_Value(const double &length) const;

	double Tolerance_Item_D_Position(const double &tol_a,const double &tol_b,const double &tol_c,const double &tol_d) const;  // Position where tolerance hits limit

	inline double Tolerance_Item_D_Position(void) const
	{
		return this->Tolerance_Item_D_Position(d_tolerance_a,d_tolerance_b,d_tolerance_c,d_tolerance_d);
	}

// MANIPULATORS
	
	void Set_Tolerance_Expression(const QString &s);

	void Set_Tolerance_Item_A(const double &value);
	void Set_Tolerance_Item_B(const double &value);
	void Set_Tolerance_Item_C(const double &value);
	void Set_Tolerance_Item_D(const double &value);

	void Update(void);	// update tolerance expressions with current values

signals:
	void Update_Tolerance(void);

private slots:
	void Tolerance_Changed(void);

protected:

private:
	
enum TToleranceType
{
	TOLERANCE_UNKNOWN = 0,
	TOLERANCE_BAND,
	TOLERANCE_FORMULA,
	TOLERANCE_FORMULA_WITH_LIMIT
};

	QLineEdit 							*d_tolerance_edit;
	
	TToleranceType						d_tolerance_type;
	double								d_tolerance_a;
	double								d_tolerance_b;
	double								d_tolerance_c;
	double								d_tolerance_d;

	void Update_Display(void);

// NOT IMPLEMENTED
	TToleranceWidget(const TToleranceWidget&);
	TToleranceWidget& operator=(const TToleranceWidget&);
};

#endif

